﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PuferFishContracts.Enums;
using PuferFishContracts.Extensions;
using PuferFishContracts.Infrastructure;
using PuferFishContracts.Exceptions;

namespace PuferFishContracts.DataModels;

public class SupplyDataModel(string id, ProductType productType, DateTime deliveryDate, int count, List<SupplyProductDataModel> products) : IValidation
{
    public string Id { get; private set; } = id;

    public ProductType ProductType { get; private set; } = productType;

    public DateTime DeliveryDate { get; private set; } = deliveryDate;

    public int Count { get; private set; } = count;

    public List<SupplyProductDataModel> Products { get; private set; } = products;
    public void Validate()
    {
        if (Id.IsEmpty())
            throw new ValidationException("Field Id is empty");
        if (!Id.IsGuid())
            throw new ValidationException("The value in the field Id is not a unique identifier");
        if (ProductType == ProductType.None)
            throw new ValidationException("Field ProductType is empty");
        if (Count <= 0)
            throw new ValidationException("Field Count is less than or equal to 0");
        if ((Products?.Count ?? 0) == 0)
            throw new ValidationException("The sale must include products");
    }
}