﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PuferFishContracts.DataModels;
using PuferFishContracts.Exceptions;

namespace PuferFishTests.DataModelsTests;

[TestFixture]
internal class ShopProductDataModelTests
{
    [Test]
    public void ShopIdIsNullOrEmptyTest()
    {
        var shopProduct = CreateDataModel(null, Guid.NewGuid().ToString(), 1);
        Assert.That(() => shopProduct.Validate(), Throws.TypeOf<ValidationException>());

        shopProduct = CreateDataModel(string.Empty, Guid.NewGuid().ToString(), 1);
        Assert.That(() => shopProduct.Validate(), Throws.TypeOf<ValidationException>());
    }

    [Test]
    public void ShopIdIsNotGuidTest()
    {
        var shopProduct = CreateDataModel("shopId", Guid.NewGuid().ToString(), 1);
        Assert.That(() => shopProduct.Validate(), Throws.TypeOf<ValidationException>());
    }

    [Test]
    public void ProductIdIsNullOrEmptyTest()
    {
        var shopProduct = CreateDataModel(Guid.NewGuid().ToString(), null, 1);
        Assert.That(() => shopProduct.Validate(), Throws.TypeOf<ValidationException>());

        shopProduct = CreateDataModel(Guid.NewGuid().ToString(), string.Empty, 1);
        Assert.That(() => shopProduct.Validate(), Throws.TypeOf<ValidationException>());
    }

    [Test]
    public void ProductIdIsNotGuidTest()
    {
        var shopProduct = CreateDataModel(Guid.NewGuid().ToString(), "productId", 1);
        Assert.That(() => shopProduct.Validate(), Throws.TypeOf<ValidationException>());
    }

    [Test]
    public void CountIsLessOrZeroTest()
    {
        var shopProduct = CreateDataModel(Guid.NewGuid().ToString(), Guid.NewGuid().ToString(), 0);
        Assert.That(() => shopProduct.Validate(), Throws.TypeOf<ValidationException>());
        shopProduct = CreateDataModel(Guid.NewGuid().ToString(), Guid.NewGuid().ToString(), -1);
        Assert.That(() => shopProduct.Validate(), Throws.TypeOf<ValidationException>());
    }

    [Test]
    public void AllFieldsIsCorrectTest()
    {
        var shopId = Guid.NewGuid().ToString();
        var productId = Guid.NewGuid().ToString();
        var count = 1;
        var shopProduct = CreateDataModel(shopId, productId, count);
        Assert.That(() => shopProduct.Validate(), Throws.Nothing);
        Assert.Multiple(() =>
        {
            Assert.That(shopProduct.shopId, Is.EqualTo(shopId));
            Assert.That(shopProduct.ProductId, Is.EqualTo(productId));
            Assert.That(shopProduct.Count, Is.EqualTo(count));
        });
    }

    private static ShopProductDataModel CreateDataModel(string? shopId, string? productId, int count) => new(shopId, productId, count);
}

